// utility class, fast ring buffer

public class FastRingBuffer extends Object{

	private int size;
	private int head;
	private int tail;
	private int objects;
	private Object ring[];

	public FastRingBuffer (int size){
		this.size = size;
		ring = new Object[size];
		head = 0;
		tail = 0;
		objects = 0;
	}


	public synchronized void push(Object object){
		if((head != tail) || (objects == 0)){
			ring[head] = object;
			if(head == size-1)
				head = 0;
			else
				head++;
				
			objects++;
		}
	}


	public synchronized Object pop(){
		Object obj = null;
		if(objects != 0){
			obj = ring[tail];
			if(tail == size-1)
				tail = 0;
			else
				tail++;
				
			objects--;
		}
		return(obj);
	}


   public Object get(int index){
   	if(index < objects){
	   	int p = tail + index ;
   		if(p > (size - 1)){
				p -= size ;			
   		}
	   	return (ring[p]);
   	}
   	else
   		return null;
   }

   
	public int capacity(){
		return (size);
	}


	public int size(){
		return objects;
	}

	
	public static void main(String arg[]){
		FastRingBuffer r = new FastRingBuffer(3);
		System.out.println("Capacite ="+r.capacity());
		System.out.println("Taille ="+r.size());
		System.out.println("Pop a vide: "+r.pop());
		r.push("1");
		System.out.println("Pop: "+r.pop());
		r.push("1");
		r.push("2");
		r.push("3");
		r.push("4");
		System.out.println("Get(1) = "+r.get(1));
		System.out.println("Taille = "+r.size());
		for(int i=0;i<4;i++)
			System.out.println("Pop: "+r.pop());

		r.push("1");
		r.push("2");
		r.push("3");
		r.pop();
		r.push("4");
		for(int i=0;i<4;i++)
			System.out.println("Get("+i+") = "+r.get(i));
			
		for(int i=0;i<4;i++)
			System.out.println("Pop: "+r.pop());
			
	}
// end of class FastRingBuffer
}
