// Class   : FlexLabel
// Author  : Russ Ethington
// Version : 1/28/96 for Java 1.0
// Notice  : Copyright (C) 1995, 1996 Russ Ethington

// Rev 1.01	 04/06/1996 by Patrice Godard
// adapted to JDK 1.02
// move to package util.ui

// Rev 1.02 05/06/1996 by Patrice Godard
// fixed bug in width calculation, prior to display

//package util.ui;

import java.awt.*;

public class FlexLabel extends Panel
{
   private String text = "";
   private Font font = null;
   private FontMetrics metrics = null;
   private Color color = Color.black;
   private int width = 0;
   private int height = 0;
   private int etching = 0;
   private boolean border = false;
   
   public static final int FlatText = 0;
   public static final int RaisedText = 1;
   public static final int LoweredText = 2;
      
   public FlexLabel() { this(""); }
   
   public FlexLabel(String theText) 
   { 
      setText(theText); 
   }
   
   public FlexLabel(FlexLabel template) 
   { 
      copyTemplate(template); 
   }
   
   public FlexLabel(String theText, FlexLabel template)
   {
      copyTemplate(template);
      setText(theText);
   }
   
   public void copyTemplate(FlexLabel template)
   {
      text = template.text;
      font = template.font;
      metrics = template.metrics;
      color = template.color;
      width = template.width;
      etching = template.etching;
      border = template.border;
   }
   
   public void checkFont()
   {
      if (font == null) font = new Font("Helvetica", Font.PLAIN, 12);
   }

   public void setText(String theText) 
   { 
      checkFont();
      text = theText;
      FontMetrics metrics = getFontMetrics(font);
      width = metrics.charsWidth(theText.toCharArray(), 0, theText.length());
      repaint();
   }
   
   public void setFont(String name, int style, int size)
   {
      font = new Font(name, style, size);  
   }
   
   public void setWidth(int theWidth) { width = theWidth; }

   public void setHeight(int theHeight) { height = theHeight; }

   public void setColor(Color theColor) { color = theColor; }
   
   public void setEtching(int theEtching) { etching = theEtching; }

   public void setBorder(boolean theBorder) { border = theBorder; } 
   
   public String getText() { return text; } 

   public Font getFont() { return font; }
   
   public int getWidth() { return width; }
   
   public Color getColor() { return color; }

   public int getEtching() { return etching; }
   
   public boolean getBorder() { return border; }
   
   public void paint(Graphics g) { update(g); }
   
   public void update(Graphics g)
   {
      Color save = g.getColor();      
      g.clearRect(0, 0, bounds().width, bounds().height);
      if (border)
      {
         g.setColor(Color.lightGray);
         g.draw3DRect(0, 0, width +5, height+5, false);
      }
      drawText(g);      
      g.setColor(save);
   }

   private void drawText(Graphics g)
   {   
      if (text == null) return;
      
      char characters[] = text.toCharArray();
      int theLength = text.length();
      Color save = g.getColor();
      
      g.setFont(font);

      FontMetrics metrics = getFontMetrics(font);
     
      

      if (etching == RaisedText)
      {
         g.setColor(Color.white);
         g.drawChars(characters, 0, theLength, 1, metrics.getHeight() - 3);
      }
      else if (etching == LoweredText)
      {
         g.setColor(Color.white);
         g.drawChars(characters, 0, theLength, 3, metrics.getHeight() - 1);
      }
      g.setColor(color);      
      g.drawChars(characters, 0, theLength, 2, metrics.getHeight() - 2);
      g.setColor(save);
   }
   
   public Dimension minimumSize() 
   {  
      if (height == 0)
      {
         if (font != null)
            height = font.getSize();
         else
            height = 12;
      }
      if (width == 0) width = 100;

      return(new Dimension(width + 3, height + 6));
   }
   
   public Dimension preferredSize() { return minimumSize(); }

   public Insets insets() { return new Insets(3, 3, 1, 1); }   
}
