/* PlayField
*/
import java.awt.*;
import Star;
import Player;
import GameOverException;
import Sounder;


public class PlayField extends Canvas implements Runnable{

	public static int WIDTH = 45; // grid size
	public static int HEIGHT = 37;

	public static int X_OFFSET = 5; // offset due to playfield's border
	
	public static int CELL_WIDTH = 10;
	public static int CELL_HEIGHT = 10;

	// grid occupancy
	public static int FREE=0;
	public static int STAR=1;
	public static int WALL=2;
	public static int SNAKE=3;

	public static int SPEED = 90; // pause between steps

   // game states
	private static int STOPPED = 0;
	private static int PLAYING = 1;
	private static int PAUSED = 2;

	// number of players
	private static int NUM_PLAYERS = 2;
	private static int NUM_PLAYERS_MAX = 2;

	// Growing factor when eating a star
	private static double GROWING_FACTOR = 2.5;

	// Last Level
	public static int LAST_LEVEL = 10;
	// Starting Level
	int level = 1;

	MediaTracker tracker;
	Image starfield;       // star field background
	Image backgroundImage; // background + decors
	Image offscreenImage;  // image for double buffering
	Image wall;				  // wall image
	Image titleImage;			  // title image
	Image dialogImage;			  // dialog background image
	Graphics offscreenGraphics = null; // graphics for double buffering
	Graphics graphics = null;
	
	Nibbles parent;
	Thread engine;
	int[][] grid;
	Image[] starImage;
	Image[] snakeImage;
	private int status = STOPPED;
	Star star = null;
	Snake snakeArray[];
	PlayField game;
	Player player[];
	LevelInstaller levelInstaller = null;
	Sounder sounder; // sounder thread, plays audioclips on demands


	
	public PlayField(Nibbles parent, Player player[]){
		super();
		reshape(0,0,200,100);
		this.parent = parent;
		this.player = player;
		tracker = new MediaTracker(this);
		game = this;

      // Load the background image.
      starfield = loadImage("gfx/stars.gif");
      tracker.addImage(starfield, 0);
      parent.showStatus("Loading Background Image...");
      
      
		// Load star Images
        int numStars = 2;
        starImage = new Image[numStars+1];
        for(int i=1;i<=numStars;i++){
        		String image = null;
        		switch(i){
        			case 1 :
        				image = new String("yelstar");
        				break;
        			case 2 :
        				image = new String("redstar");
        				break;
        		}
        		if(Nibbles.DEBUG != 0)
        			System.out.println("Loading image gfx/"+image+".gif");
        		starImage[i] = loadImage("gfx/"+image+".gif");
            tracker.addImage(starImage[i], i);
            
        }
        parent.showStatus("Loading Star Images...");

        // Load wall image
  		  wall = loadImage("gfx/wall.gif");
		  if(Nibbles.DEBUG !=0)
					System.out.println("Loading gfx/wall.gif");
  		  tracker.addImage(wall, numStars+1);
  		  parent.showStatus("Loading Wall Image...");
        
        // Load snake Images
        int numSnakes = 2;
        snakeImage = new Image[numSnakes+1];
		  for(int i=1;i<=numSnakes;i++){
        		if(Nibbles.DEBUG !=0)
					System.out.println("Loading "+"snake"+i+".gif");
        		snakeImage[i] = loadImage("gfx/snake"+i+".gif");
            tracker.addImage(snakeImage[i], i+numStars+1);
        }
        parent.showStatus("Loading Snakes Images");

        // load Title and Dialog background images
        if(Nibbles.DEBUG !=0)
					System.out.println("Loading gfx/title.gif");
        titleImage = loadImage("gfx/title.gif");
        tracker.addImage(titleImage,numStars+numSnakes+2);
        parent.showStatus("Loading Title Image...");

		  if(Nibbles.DEBUG !=0)
					System.out.println("Loading gfx/FondDialNib.gif");
        dialogImage = loadImage("gfx/FondDialNib.gif");
        tracker.addImage(dialogImage,numStars+numSnakes+3);
        parent.showStatus("Loading Dialog's Background Image");

        
        try{
   	  			tracker.waitForAll();
        }catch(InterruptedException e) {System.out.println(e);}
        
        parent.showStatus("Ready For Playing!");
        
        backgroundImage = createImage(starfield.getSource());

        sounder = new Sounder(parent);        

        snakeArray = new Snake[NUM_PLAYERS_MAX];
        status = STOPPED;
             
   }


   /** init the playfield
   */
   private void initPlayField(){
		initGrid(0); // init grid
		Star.reset(); // init star number
		star = null; // no star

		offscreenImage = createImage(starfield.getWidth(null),starfield.getHeight(null));
		offscreenGraphics = offscreenImage.getGraphics();  // init offscreenGraphics
		offscreenGraphics.drawImage(backgroundImage,0,0,this); // draw background

		// init decors here
		if(levelInstaller == null)
			levelInstaller = new LevelInstaller(wall);
			
		levelInstaller.initLevel(offscreenGraphics, grid, level);
		star = newStar(true);  // first star

		snakeArray[0] = newSnake(snakeImage[1],levelInstaller.snake1Pos(level).x,levelInstaller.snake1Pos(level).y,levelInstaller.snake1Dir(level).x,levelInstaller.snake1Dir(level).y); // test
		if(NUM_PLAYERS > 1)
			snakeArray[1] = newSnake(snakeImage[2],levelInstaller.snake2Pos(level).x,levelInstaller.snake2Pos(level).y,levelInstaller.snake2Dir(level).x,levelInstaller.snake2Dir(level).y); // test

   }


	/** init the grid
	*/
   private void initGrid(int level){
	   grid = new int[WIDTH][HEIGHT];
	   // tester suivant level pour installer le decor a incruster ds
	   // l'image de fond
   }


	

   /** place a star with corresponding number on
   * a free place on the grid
   */
   private Star newStar(boolean isFirst){
   	int x, y;
	   do{
	   	 x = (int) (java.lang.Math.random()*WIDTH);
	   	 y = (int) (java.lang.Math.random()*HEIGHT);
	   }while(grid[x][y] != FREE);
	   grid[x][y] = STAR;

		Star tmpStar;
		if(isFirst)
			tmpStar = new Star(offscreenGraphics,starImage[1], x, y);
		else
			tmpStar = new Star(offscreenGraphics,star.number() < 9  ? starImage[1]:starImage[2], x, y);
			
		if(Nibbles.DEBUG !=0)
			System.out.println("adding a new star number "+tmpStar.number()+" at "+x+","+y);
		
	   return(tmpStar);
   }


	/** create a new Snake
	*/
	private Snake newSnake(Image snakeImage, int sx, int sy, int dx, int dy){
	
	   grid[sx][sy] = SNAKE;
	   
	   return (new Snake(this, snakeImage, sx, sy, dx, dy));
	}


	/** mark a grid cell free
	*/
	public void freeGrid(int x, int y){
		grid[x][y] = FREE;	
	}

	
	/** main Thread method
	*/
	public synchronized void run(){
		int dead = 0;
		repaint(); // repaints all playfield before the game starts
		
		try{
			
		while(true){
		//	System.out.println("Engine Running....");
		for( int i = 0; i < NUM_PLAYERS; i++){
			  snakeArray[i].creep();
		}
	   repaint();

	   // increment scores
	   incScore(player[0], snakeArray[0]);
	   if(NUM_PLAYERS > 1)
		   incScore(player[1], snakeArray[1]);	

	   // collision detection
	   	
	   Point p;
	   for(int i = 0; i < NUM_PLAYERS; i++){
			  p = snakeArray[i].getPosition();
			  if(grid[p.x][p.y] >= WALL){ //if wall or snake
			  		dead+=(i+1);
			  }
			  else if (grid[p.x][p.y] == STAR){ // eat a star
			  		snakeArray[i].eat((int)((double)star.number()*GROWING_FACTOR));

			  		// check if last star
			  		if(star.number() == 10){
			  			sounder.play(Sounder.LEVEL);
			  			if(level < LAST_LEVEL)
				  			level++;
				  		else
				  			level = 1;
				  			
						levelMessage();
						try{
							wait(2000);
				  		}catch(InterruptedException e){System.out.println(e);}
				  		player[i].extraLife();	
			  			initPlayField();
			  			
			  			p = snakeArray[i].getPosition();
			  			break;
			  		}
			  		// check if last level
			  		else{
			  			sounder.play(Sounder.STAR);
				  		star = newStar(false);
			  		}
			  }
			  grid[p.x][p.y] = SNAKE;
	   }
	   
	   // "frontal shock" test
	   if(NUM_PLAYERS > 1)
	   	if(snakeArray[0].getPosition().equals(snakeArray[1].getPosition()))
	   		dead = 3;

			//lose a life and check for game over
	   if(dead != 0){

	   	sounder.play(Sounder.DEAD);
	   
			switch (dead) {
				case 1 :   if(Nibbles.DEBUG != 0)
								  System.out.println("Player 1 died");
							  message("Player 1 died!");
							  player[0].loseLife();
							  break;

				case 2 :	  if(Nibbles.DEBUG != 0)
									System.out.println("Player 2 died");
							  player[1].loseLife();
							  message("Player 2 died!");
							  break;

				case 3 :	  if(Nibbles.DEBUG != 0)
								  System.out.println("Both Players died");
			 			 	  player[0].loseLife();
			 			 	  player[1].loseLife();
			 			 	  message("Both Players died!");
							  break;
			}
			if(player[0].gameOver() || player[1].gameOver())
				throw new GameOverException();

			try{
				wait(3000);
   		}catch(InterruptedException e){System.out.println(e);}
   		
			initPlayField();
			dead = 0;
			repaint();
			
			//force GC
			/* disabled, crashed netscape! :-(
			System.gc();
			System.runFinalization();
			*/
		}
   	  	   
			try{
				wait(SPEED);
			}catch(InterruptedException e){System.out.println(e);}
		}
		}catch(GameOverException e) {
			System.out.println("GAME OVER!");
			if(player[0].gameOver() && !player[1].gameOver()){
				message ("Player 2 Wins!");
				if(Nibbles.DEBUG != 0)
					System.out.println("Player 2 Wins!");
			}
			else if(!player[0].gameOver() && player[1].gameOver()){
				message ("Player 1 Wins!");
				if(Nibbles.DEBUG != 0)
					System.out.println("Player 1 Wins!");
			}
			else{
				message ("Draw Game!");
				if(Nibbles.DEBUG != 0)
					System.out.println("Draw Game!");
			}
			}
			stop();
	}
	

	private void incScore(Player player, Snake snake){
		if(snake.isGrowing())
			player.incScore(100);
		else
			player.incScore(5);
	}

	
	public void play(){
		if(Nibbles.DEBUG !=0)
			System.out.println("PlayField.play() called");
			
		requestFocus();
		
		if(status == STOPPED){
			parent.enablePlayerSelection(false); // disable nb of players selection during game
			initPlayField();
			player[0].init();
			player[1].init();
			sounder.play(Sounder.LEVEL);
			engine = new Thread(this);
			status = PLAYING;
			engine.start();
		}
		else if(status == PAUSED){
			status = PLAYING;
			engine.resume();
		}
	}


	public void pause(){
		if(Nibbles.DEBUG !=0)
			System.out.println("PlayField.pause() called");
		
		if(status == PLAYING){
			status = PAUSED;
			engine.suspend();
		}
		else if(status == PAUSED){
			requestFocus();
			status = PLAYING;
			engine.resume();
		}
		
	}
	
	
	public void stop(){
		if(Nibbles.DEBUG !=0)
			System.out.println("PlayField.stop() called");

		if(status != STOPPED){
			parent.enablePlayerSelection(true); // enable nb of players selection
			engine.stop();
			status = STOPPED;
			engine = null;
			level = 1;
			if(Nibbles.DEBUG !=0)
			System.out.println("Stopping Game");
		}
	}

	
   public Dimension minimumSize(){
   	return new Dimension(460,390);
   }


   public Dimension preferredSize(){
   	return minimumSize();
   }


	public void paint(Graphics g) {
		if(Nibbles.DEBUG > 100)
			System.out.println("PlayField.paint() called");
  		update(g);
	}


	public boolean keyDown(Event e, int key){
		Snake snake1 = (Snake) (snakeArray[0]);
		Snake snake2 = null;
		if(NUM_PLAYERS > 1)
		{
			snake2 = (Snake) (snakeArray[1]);
		switch(key){
			case '8' : snake2.changeDirection(0,1);
						  if(Nibbles.DEBUG >= 180)
						  		System.out.println("Direction Snake 0 = 0, 1");
						  break;

		  case '5' : snake2.changeDirection(0,-1);
						  if(Nibbles.DEBUG >= 180)
						  		System.out.println("Direction Snake 0 = 0, -1");
						  break;

		  case '6' : snake2.changeDirection(1,0);
						  if(Nibbles.DEBUG >= 180)
						  		System.out.println("Direction Snake 0 = 1, 0");
						  break;

	     case '4' : snake2.changeDirection(-1,0);
		 				  if(Nibbles.DEBUG >= 180)
						  		System.out.println("Direction Snake 0 = -1, 0");
						  break;
		}
		}
		
		switch(key){
		  case 'e' : snake1.changeDirection(0,1);
						  if(Nibbles.DEBUG >= 180)
						  		System.out.println("Direction Snake 1 = 0, 1");
						  break;

		  case 'd' : snake1.changeDirection(0,-1);
						  if(Nibbles.DEBUG >= 180)
						  		System.out.println("Direction Snake 1 = 0, -1");
						  break;

		  case 'f' : snake1.changeDirection(1,0);
						  if(Nibbles.DEBUG >= 180)
						  		System.out.println("Direction Snake 1 = 1, 0");
						  break;

	     case 's' : snake1.changeDirection(-1,0);
		 				  if(Nibbles.DEBUG >= 180)
						  		System.out.println("Direction Snake 1 = -1, 0");
						  break;
						  
		}
		return(true);
	}

	
	/** * update method
	* implements double-buffering
	* and smart redraw
	* well... at least strives to!
	*/
	public void update(Graphics g){

		Rectangle clipRect = g.getClipRect(); // get clipping rectangle
		
		if(Nibbles.DEBUG > 200)
			System.out.println("PlayField.update() called, clipRect= "+
										clipRect.x+", "+clipRect.y+", "+
										clipRect.width+", "+clipRect.height);

  	 	if(offscreenImage == null){ // if not initialized
  	 		g.drawImage(backgroundImage,0,0,this);
  	 		graphics = g;
  	 		Rectangle r = bounds();
			g.drawImage(titleImage,(r.width/2)-(titleImage.getWidth(null)/2),
							(r.height/2)-(titleImage.getHeight(null)/2),null);
  	 	}
  	 	else
			g.drawImage(offscreenImage,0,0,this);
	}


	/** *fill given rectangle with
	    *background image
	*/
	public void clear(Rectangle rect){
	//	System.out.println("PlayField.clear() called : "+rect);
		offscreenGraphics.setColor(Color.black);
	//	offscreenGraphics.clipRect(rect.x, rect.y, rect.width, rect.height);
		offscreenGraphics.fillRect(rect.x, rect.y, rect.width, rect.height);
    //offscreenGraphics.drawImage(backgroundImage,0,0,this);
	
		
	}


	public void message(String message){
		Rectangle r = bounds();
		int width = dialogImage.getWidth(null);
		int height = dialogImage.getHeight(null);
		FontMetrics fm;
		int messageWidth;
		int messageHeight;
		Font font;
		int baseX;
		int baseY;

		baseX = (r.width/2)-(width/2);
		baseY = (r.height/2)-(height/2);
		
		offscreenGraphics.drawImage(dialogImage,baseX,baseY,null);
		
		font = new Font("Helvetica", Font.BOLD,20);		
		offscreenGraphics.setFont(font);
		offscreenGraphics.setColor(Color.white);
		
		fm = graphics.getFontMetrics();
		messageWidth = fm.stringWidth(message);
		messageHeight = fm.getHeight();	

		offscreenGraphics.drawString(message, baseX+ (width/2)-(messageWidth/2)-20, baseY+ (height/2)+5);
	
		repaint();
	}


	public void levelMessage(){
		message("Level " + level);
	}
	
   /** *Utility method to load an Image,
       *from an Applet or Application
   */
    public Image loadImage(String path){
    	if (parent.inAnApplet()) //Load with Applet getImage(url, string) method.
            return parent.getImage(parent.getCodeBase(), path);
            
      else   //Load with Toolkit getImage(file) method.
            return parent.getToolkit().getImage(path);
    }
 
 
 public void setNbPlayers(int n)
 {
 	if(Nibbles.DEBUG !=0)
			System.out.println("PlayField.setNbPlayers() called. Setting number of players to "+n);
	
	NUM_PLAYERS = n;
	
 }
	
// end of class PlayField
}